# HRK 2012/175
#
# lib to be used for: UVFITSFG_2_FLAGCMD_TXT.py
#

# Authors: 

# Hans - Rainer Kloeckner
# MPIfR / Oxford
# hrk@mpifr-bonn.mpg.de
# hrk@astro.ox.ac.uk


# Copyright (c) 2012
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>,
# or write to the Free Software Foundation, Inc., 
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


def datatype(data):
    """
    return the datatype of the file
    """
    from AIPSData import AIPSCat
    from string import split,find
    #
    datatype = 'None'
    
    catalog_in = AIPSCat(int(data[3]))
    catalog    = str(catalog_in).split()
    #
    matchfname = []
    for i in range(len(catalog)):
        if catalog[i].find(data[0]) == 0:
                matchfname.append(i)
    matchfname.append(len(catalog))

    classindex = -1
    isname  = -1
    isclass = -1
    isdisk  = -1
    isseq   = -1
    i = 0
    while (isname != 1 or isclass != 1 or isdisk !=1 or isseq !=1 or classindex != -1) and i < len(matchfname)-1:
        isname  = -1
        isclass = -1
        isdisk  = -1
        isseq   = -1
        #print matchfname[i],matchfname[i+1]
        #print catalog[matchfname[i]:matchfname[i+1]]
        for j in range(matchfname[i],matchfname[i+1]):

            if catalog[j].replace('.','') == data[0].replace('.',''):
                isname = 1
            if catalog[j].replace('.','') == data[1]:
                isclass = 1
            if catalog[j].replace('.','') == str(int(data[2])):
                isdisk = 1
            if catalog[j].replace('.','') == str(int(data[3])):
                isseq = 1
                        
            if catalog[j] == 'UV' and isname == 1 and isclass == 1 and isdisk == 1 and isseq ==1 and classindex == -1:
                classindex = j
            if catalog[j] == 'MA' and isname == 1 and isclass == 1 and isdisk == 1 and isseq ==1 and classindex == -1:
                classindex = j
        i += 1

    if classindex != -1:
        datatype = catalog[classindex]

    return(datatype)


def givestokes(data):
    """
    provides you with the stokes parameter 
    this takes into account dual freq observations
    taken with the GMRT
    610 MHz LSB + USB  RR
    240 MHz       USB  LL  (need to check)
    """
    from AIPSData import AIPSUVData as UVDataS
    uvdata  = UVDataS(data[0],data[1],data[2],data[3])
    obsfreq = uvdata.header.crval[2]
    nstokes = getheader(data,'STOKES')[3]
    tele    = uvdata.header.telescop
    st = []
    if nstokes == 1 and obsfreq > 250E6:
	st.append('RR')
    elif nstokes == 1 and obsfreq < 250E6 and tele == 'GMRT':
	st.append('LL')
    else:
        st = uvdata.stokes
    #
    if len(st) == 0:
	print '\nCAUTION could not determine STOKES parameter (givestokes)\n'
	sys.exit(-1)
    return(st)

def getheader(data,gethead='FREQ'):
	"""
	returns the value, referenc, increment, dimention of the 
	image of the header mostly used to get
        e.g. 'RA', 'DEC','STOKES', 'FREQ','IF'

        output would be e.g. for FREQ [61400000,0.5,125000.0,128]
        """
        #
        if  datatype(data) == 'UV':
            from Wizardry.AIPSData import AIPSUVData as CHECKHEADER
        else:
            from Wizardry.AIPSData import AIPSImage as CHECKHEADER
        #
        chdata   = CHECKHEADER(data[0],data[1],int(data[2]),int(data[3]))
        chaxis   = chdata.header['ctype']
	chval    = -1
	chref    = -1
	chincr   = -1
	chdim    = -1

        for i in range(len(chaxis)):
                if chaxis[i] == gethead:
                        chval  = chdata.header['crval'][i]
                        chref  = chdata.header['crpix'][i]
                        chincr = chdata.header['cdelt'][i]
                        chdim  = chdata.header['naxis'][i]	
		
        return[chval,chref,chincr,chdim]

def getntable(data,inext='CC'):
        """
        returns the number of speficied AIPS table 
        """
        if  datatype(data) == 'UV':
            from AIPSData import AIPSUVData as GETTAB
        else:
            from AIPSData import AIPSImage as GETTAB
        #
        tadata   = GETTAB(data[0],data[1],int(data[2]),int(data[3]))
        tables = tadata.tables
        ntable = -1
        for i in range(len(tables)):
                if tables[i][1] == 'AIPS '+inext:
                        ntable = tables[i][0]
        return(int(ntable))


def datatobase(data,IF=0,doheader=False):
	"""
        construct the visibilities into a python dictionary 
        with baselines as a first key
	"""
	from numpy import array
	from math import sqrt
	from copy import copy

	if  datatype(data) == 'UV':
		from Wizardry.AIPSData import AIPSUVData as AIPSDataVIS
	else:
		print 'Data is not UV data',data,' abord.'

	stokes = givestokes(data)

	visdata = AIPSDataVIS(data[0],data[1],data[2],data[3])

	print '\n- loading the data into memory\n'

	hi = {}
	hi['HEADER'] = copy(visdata.header)

	h = {}
	for visibility in visdata:
		for st in range(len(stokes)):

			if isnanv(array(visibility.visibility[IF,:,st,1]).flatten()) == False:

				if h.has_key(str(visibility.baseline)) == False:
					h[str(visibility.baseline)] = {}

				if h.has_key(str(visibility.baseline)) == True:

					if h[str(visibility.baseline)].has_key(str('time'+str(IF)+str(stokes[st]))) == True:
						h[str(visibility.baseline)]['time'+str(IF)+str(stokes[st])].append(visibility.time)
					else:
						h[str(visibility.baseline)][str('time'+str(IF)+str(stokes[st]))] = [visibility.time]


					if h[str(visibility.baseline)].has_key(str('real'+str(IF)+str(stokes[st]))) == True:
						h[str(visibility.baseline)]['real'+str(IF)+str(stokes[st])].append(array(visibility.visibility[IF,:,st,0]))
					else:
						h[str(visibility.baseline)][str('real'+str(IF)+str(stokes[st]))] = [array(visibility.visibility[IF,:,st,0])]


					if h[str(visibility.baseline)].has_key(str('imag'+str(IF)+str(stokes[st]))) == True:
						h[str(visibility.baseline)]['imag'+str(IF)+str(stokes[st])].append(array(visibility.visibility[IF,:,st,1]))
					else:
						h[str(visibility.baseline)][str('imag'+str(IF)+str(stokes[st]))] = [array(visibility.visibility[IF,:,st,1])]


					if h[str(visibility.baseline)].has_key(str('weig'+str(IF)+str(stokes[st]))) == True:
						h[str(visibility.baseline)]['weig'+str(IF)+str(stokes[st])].append(array(visibility.visibility[IF,:,st,2]))
					else:
						h[str(visibility.baseline)][str('weig'+str(IF)+str(stokes[st]))] = [array(visibility.visibility[IF,:,st,2])]

					if h[str(visibility.baseline)].has_key(str('uvdis'+str(IF)+str(stokes[st]))) == True:
						h[str(visibility.baseline)]['uvdis'+str(IF)+str(stokes[st])].append(sqrt(visibility.uvw[0]**2+visibility.uvw[1]**2+visibility.uvw[2]**2))
					else:
						h[str(visibility.baseline)][str('uvdis'+str(IF)+str(stokes[st]))] = [sqrt(visibility.uvw[0]**2+visibility.uvw[1]**2+visibility.uvw[2]**2)]


					if h[str(visibility.baseline)].has_key(str('uvw'+str(IF)+str(stokes[st]))) == True:
						h[str(visibility.baseline)]['uvw'+str(IF)+str(stokes[st])].append([visibility.uvw[0],visibility.uvw[1],visibility.uvw[2]])
					else:
						h[str(visibility.baseline)][str('uvw'+str(IF)+str(stokes[st]))] = [[visibility.uvw[0],visibility.uvw[1],visibility.uvw[2]]]


	if doheader == True:
		return(h,hi)
	else:
		return(h)

def dbaseinfo(database):
        """
        Provide database info
        """
        bsl = database.keys()
        ifs    = -1
        stokes = []
        for bl in range(len(bsl)):
                blkeys = database[bsl[bl]].keys()
                for blk in range(len(blkeys)):
                        dinfo = blkeys[blk].replace('imag','').replace('uvw','').replace('weig','').replace('uvdis','').replace('time','').replace('real','')
                        if dinfo.count('RR') > 0:
                                if stokes.count('RR') == 0:
                                        stokes.append('RR')
                                ifs = int(dinfo.replace('RR',''))

                        if dinfo.count('LL') > 0:
                                if stokes.count('LL') == 0:
                                        stokes.append('LL')
                                ifs = int(dinfo.replace('LL',''))
        return(bsl,stokes,ifs)


def radec2deg(rah,ramin,rasec,degh='None',degmin='None',degsec='None'):
        """
        Convert radian into degress
        """
        degh = str(degh)
        ra = float(rah)*60*60*15 + float(ramin)*60*15 + float(rasec)*15 

        if degh !='None':
                dec = abs(float(degh))*3600 + abs(float(degmin))*60 + abs(float(degsec))
                if degh[0] == '-':
                        dec = -1*dec
        else:
                dec = 0

        ra2  =ra/3600.0
        dec2 =dec/3600.0

        return(ra2,dec2)


def offset(pc,so,unit='s'):
        """
        computes the offset in arcsecounds [if units is s]
        pc phase centre [rah,ramin,rasec,degh,degmin,degsec]
        so source  [rah,ramin,rasec,degh,degmin,degsec]
        the output uses the AIPS convention
        to be used in UVSUB
        """
        from math import cos,pi
        if unit == 'd':
                ufactor = 1
        elif unit == 'm':
                ufactor = 60
        else:
                ufactor = 3600
        if len(pc) > 2:
                pcrad,pcdecd = radec2deg(pc[0],pc[1],pc[2],pc[3],pc[4],pc[5])
        else:
                pcrad  = pc[0]
                pcdecd = pc[1]

        sorad,sodecd = radec2deg(so[0],so[1],so[2],so[3],so[4],so[5])

        decshift = -1 * ufactor * (pcdecd - sodecd)                    # deg
        rashift  = ufactor * (sorad - pcrad) * cos(pcdecd*(2*pi/360.)) # deg

        return(rashift,decshift)

def uvaddsour(uvdata,sou,dosub=-1):
        """
        add/subtracts sources from the UVdataset 
        """
        from AIPSData import AIPSUVData
        from AIPSTask import AIPSTask

        data     = AIPSUVData(uvdata[0],uvdata[1],int(uvdata[2]),int(uvdata[3]))
        obsfreq  = getheader(uvdata,'FREQ')[0]
        pc       = [getheader(uvdata,'RA')[0],getheader(uvdata,'DEC')[0]]

        uvcop             = AIPSTask('uvcop')
        uvcop.indata      = data
        uvcop.outname     = 'TMP1'
        uvcop.outclass    = 'UVCCS'
        uvcop.outdisk     = int(uvdata[2])
        uvcop.msgkill     = -2
        uvcop.go()

        tmp1     = AIPSUVData('TMP1','UVCCS',int(uvdata[2]),int(uvdata[3]))

        for i in range(len(sou)):
                sradec = [sou[i][0][0],sou[i][0][1],sou[i][0][2],sou[i][0][3],sou[i][0][4],sou[i][0][5]]

                # determine offset
                # 
                if sradec[0] == sradec[1] == sradec[2] == sradec[3] == sradec[4] == sradec[5]:
                    sooffset = 0,0
                else:
                    sooffset = offset(pc,sradec)                    
                #
                uvsub              = AIPSTask('uvsub')
                uvsub.indata       = tmp1
                uvsub.outname      = 'TMP2'
                uvsub.outclass     = 'UVCCS'
                uvsub.outseq       = int(uvdata[2])
                uvsub.outdisk      = int(uvdata[3])
                uvsub.cmethod      = 'DFT'
                uvsub.factor       = dosub
                #
                if 2 >= len(sou[i]):
                        uvsub.smodel[1:]   = [sou[i][1],sooffset[0],sooffset[1],0]
                        modstg = 'Adding '
                        if sou[i][1] < 0:
                            modstg = 'Subtract'

                        print '\n'+modstg+' source: ',i+1,' of ',len(sou),'sources.\n'
                        print modstg+' point model'
                        print 'of :',sou[i],'\n'

                else:
                        uvsub.smodel[1:]   = [sou[i][1],sooffset[0],sooffset[1],1,sou[i][2],sou[i][3],sou[i][4]]
                        print '\nSubtract source: ',i+1,' of ',len(sou),'sources.\n'
                        print 'Subtract exteded source with model parameter'
                        print 'of :',sou[i],'\n'
                        #
                        # SMODEL(1) = flux density (Jy)
                        # SMODEL(2) = X offset in sky (arcsec)
                        # SMODEL(3) = Y offset in sky (arcsec)
                        # SMODEL(4) = Model type:
                        # 0 => point model
                        # 1 => elliptical Gaussian and
                        # SMODEL(5) = major axis size (arcsec)
                        # SMODEL(6) = minor axis size (arcsec)
                        # SMODEL(7) = P. A. of major axis (degrees)
                        # 2 => uniform sphere and
                        # SMODEL(5) = radius (arcsec)
                        # 3 => uniform sphere and
                        #SMODEL(5) = radius (arcsec)
                 #
                uvsub.go()
                # delete tmp1 
                tmp1.zap()
                tmp2     = AIPSUVData('TMP2','UVCCS',int(uvdata[2]),int(uvdata[3]))             
                #
                uvcop             = AIPSTask('uvcop')
                uvcop.indata      = tmp2
                uvcop.outname     = 'TMP1'
                uvcop.outclass    = 'UVCCS'
                uvcop.outseq      = int(uvdata[2])
                uvcop.outdisk     = int(uvdata[3])
                uvcop.msgkill     = -2
                uvcop.go()
                # delete file
                tmp2.zap()
                tmp1     = AIPSUVData('TMP1','UVCCS',int(uvdata[2]),int(uvdata[3]))

        uvcop             = AIPSTask('uvcop')
        uvcop.indata      = tmp1
        uvcop.outname     = uvdata[0]+'DS'
        uvcop.outclass    = uvdata[1]
        uvcop.outseq      = int(uvdata[2])
        uvcop.outdisk     = int(uvdata[3])
        uvcop.msgkill     = -2
        uvcop.go()
        #
        tmp1.zap()

        return([uvcop.outname,uvcop.outclass,int(uvcop.outseq),int(uvcop.outdisk)])


def mkmodel(orgdata,sources,IF=0):

        from AIPSTask import AIPSTask

	if  datatype(orgdata) == 'UV':
		from Wizardry.AIPSData import AIPSUVData as AIPSDataVis
	else:
		print 'Data is not UV data',orgdata,' abord.'

        uvdata = AIPSDataVis(orgdata[0],orgdata[1],orgdata[2],orgdata[3])

        uvcop             = AIPSTask('uvcop')
        uvcop.inname      =  orgdata[0]
        uvcop.inclass     =  orgdata[1]
        uvcop.inseq       = int(orgdata[2])
        uvcop.indisk      = int(orgdata[3])
        uvcop.outname     = 'S'+str(len(sources))
        uvcop.outclass    = 'MDAT'
        uvcop.flagver     = -1
        uvcop.outseq      = int(orgdata[2])
        uvcop.outdisk     = int(orgdata[3])
        uvcop.msgkill     = -2
        uvcop.go()

        mdata = [uvcop.outname,uvcop.outclass,int(uvcop.outseq),int(uvcop.outdisk)]

	stokes  = givestokes(mdata)
	visdata = AIPSDataVis(mdata[0],mdata[1],mdata[2],mdata[3])

        # here clear the original dataset
	for visibility in visdata:
		for st in range(len(stokes)):
                    visibility.visibility[IF,:,st,0] = [0]
                    visibility.visibility[IF,:,st,1] = [0] 
                    visibility.visibility[IF,:,st,2] = [1]
                    visibility.update()

        # adding the source to the "blanked" dataset
        uvmodel = uvaddsour(mdata,sources,dosub=-1)

        # deletes the model dataset
        uvdata = AIPSDataVis(mdata[0],mdata[1],mdata[2],mdata[3])
        uvdata.zap()

        return(uvmodel)


def gains2table(data,gains):
    """
    convert the gains into a CL table
    """
    from AIPSData import AIPSUVData
    from Wizardry.AIPSData import AIPSUVData as WizAIPSUVData
    from AIPSTask import AIPSTask

    soltime = []
    for i in range(len(gains)):
        soltime.append(gains[i][0])

    antk    = gains[0][1].keys()
    noant   = len(antk)

    uvdata = AIPSUVData(data[0],data[1],data[2],data[3])

    if getntable(data,'SN') == -1:
        # I need a dummy SN file
        calib           = AIPSTask('calib')
        calib.indata    = uvdata
        calib.smodel[1] = 1/120.
        calib.go()
        uvdata2 = AIPSUVData(data[0],'CALIB',data[2],data[3])
        uvdata2.zap()

    # now use the dummy SN table and copy X lines to a new one
    #
    tabed          = AIPSTask('tabed')
    tabed.indata   = uvdata
    tabed.inext    = 'SN'
    tabed.invers   = getntable(data,'SN')
    tabed.bcount   = 1
    tabed.ecount   = noant*len(soltime)
    tabed.outdata  = uvdata
    tabed.outvers  = getntable(data,'SN') + 1
    tabed.optype   = 'COPY'
    tabed.go()
    #
    uvdata   = WizAIPSUVData(data[0],data[1],data[2],data[3])
    sntab  = uvdata.table('SN',getntable(data,'SN'))
    #
    ant  = 1
    t    = 0
    j    = 1
    for row in sntab:
        if j > 1 :
            if (ant-1)%noant == 0:
                ant     = 1
                t      += 1

        #{'time_interval': 0.0, 'rate_1': 0.0, 'weight_1': 141.67694091796875, 'mbdelay1': 0.0, 'refant_1 : 1, 'antenna_no': 20, 'node_no': 0, 'freq_id': 1, 'i_far_rot': 0.0, 'source_id': 1, 'delay_1':  .0, 'time': 0.921970546245575, 'real1': -0.8876171708106995, 'subarray': 1, 'imag1': -0.46058222 5148163}

        # Bug in PT no has_key functionality
        row['time']       = gains[t][0]
        row['antenna_no'] = ant

        row['weight_1']   = 1
        row['source_id']  = 0

        row['real1'] = gains[t][1]['ANT_'+str(ant)]['RR'].real
        row['imag1'] = gains[t][1]['ANT_'+str(ant)]['RR'].imag

        keywd = True
        try:
            row['real2'] 
        except AttributeError:
            keywd = False
        
        if keywd == True:
            row['real2'] = gains[t][1]['ANT_'+str(ant)]['LL'].real
            row['imag2'] = gains[t][1]['ANT_'+str(ant)]['LL'].imag

        ant += 1
        j += 1
        row.update()

    return(sntab)


def mkimage(uvdat,datab=None,sntable=0,cccomp=1,wrobust=0,imagesize=[256,256],cellsi=[0,0]):
    """
    will make an image
    """
    from AIPSData import AIPSUVData, AIPSImage
    from AIPSTask import AIPSTask
    from math import pi
    
    uvdata = AIPSUVData(uvdat[0],uvdat[1],uvdat[2],uvdat[3])
    #
    if cellsi == [0,0]:
        obsfreq  = getheader(uvdat,'FREQ')[0]
        if datab == None:
            # load org database
            datab     = datatobase(uvdat,IF=0)
        uvdismax = getminmaxuvdis(datab)[1]
	light_c  = 299792458.0        # m s-1
	wl       = light_c/obsfreq
	fwhm     = wl/uvdismax*360.0*60.*60./(2.0*pi)
	cellsi   = [fwhm/3,fwhm/3]

    docalib = -1
    sntab   = sntable 
    if sntable == 0:
        print 'Calibration will be applied'
        sntab   = getntable(uvdat,'SN')
        docalib = 1
    #
    fstokes = 'I'
    #
    # IMAGING
    #
    imagr              = AIPSTask('imagr')
    imagr.indata       = uvdata
    imagr.stokes       = fstokes
    imagr.docalib      = docalib
    imagr.gainuse      = sntab
    imagr.doband       = -1
    imagr.flagver      = -1
    imagr.nfield       = 1
    imagr.nchav        = 10000
    imagr.cellsi[1:]   = cellsi
    imagr.imsi[1:]     = imagesize
    imagr.robust       = wrobust
    imagr.niter        = cccomp
    imagr.outname      = ''
    imagr.outdisk      = uvdat[3]
    imagr.go()
    #
    imdataba = AIPSImage(uvdat[0],fstokes+'BM001',1,uvdat[3])
    imdataba.zap()
    #
    return([uvdat[0],fstokes+'CL001',1,imagr.outdisk])


def save2disk(data,fname,doclean=-1,fnamext='.FITS'):
        """
        save the aips file into the current directory
        the filename: will be build via fname+file2image+fnamext
        doclean = 1 will delete file from AIPS DISK
        """
        from AIPSData import AIPSUVData, AIPSImage
        from AIPSTask import AIPSTask
        from os import getcwd,system
        #
        file2save = fname+fnamext

        if len(filesindir(file2save)) > 0:
            file2save = fname+'.'+str(len(filesindir(file2save)))+fnamext

        #
        # Save final tables to disk
        #
        fittp              = AIPSTask('fittp')
        fittp.inname       = data[0]
        fittp.inclass      = data[1]
        fittp.indisk       = int(data[2])
        fittp.inseq        = int(data[3])
#       fittp.dataout      = 'PWD:'+file2save
        fittp.dataout      = 'PWD:DATATOSAVE.FITS'
        fittp.go()
        #
        # prevent the AIPS limitation of file name lengh
        system('mv DATATOSAVE.FITS '+file2save)

        cwd = getcwd()
        if checkfi(cwd+'/'+file2save)  == 1:
                if doclean > 0:
                    if  datatype(data) == 'UV':
                        from AIPSData import AIPSUVData as ZAPData
                    else:
                        from AIPSData import AIPSImage as ZAPData
                    data2zap = ZAPData(data[0],data[1],int(data[2]),int(data[3]))
                    data2zap.zap()
        #
        return(checkfi(cwd+'/'+file2save))



# Some stupied python crap
#
def isNaN(x):
    return x != x

def isnan(x):
    """
    return a false or true if nan or NaN
    """
    from numpy import isnan as npisnan 
    #isn1 = isinstance(x,float) 
    isn2 = isNaN(x)
    isn3 = npisnan(x)
#    print isn1,isn2,isn3

    #if isn1 == True or isn2 == True or isn3 == True:
    if isn2 == True or isn3 == True:
        return True
    else:
        return False

def newisnanv(x):
    """
    return a false or true if nan or NaNval

    math isnan is a new function in python 2.6 
    so need to keep the old stuff for the time being
    """
    from math import isnan
    nan = False
    for i in range(len(x)):
        if isnan(x[i]) == True:
            nan = True
            break
    return nan

def isnanv(x):
    """
    return a false or true if nan or NaN
    """
    from numpy import array
    nan = False
    for i in range(len(x)):
        ix = array(x[i]).flatten()
        for j in range(len(ix)):
            if isnan(ix[j]) == True:
                nan = True
                break
    return nan


def eigval(nant=30,stokes='RR'):
    """
    HRK - Dummy function for developing

    code = evcal(nant, ne, toliter, model, vt, gains, k)

    and for this to work, the function definition on line 54 (def evcal(na,

    ne, toliter, Vs, V, gains, k)) must have been set up.
    """
    eigs = [-0.08581736+0.05747998j, 0.12566155-0.11368559j,0.06209708-0.14734123j,-0.30349152+0.0973056j,0.16118845+0.01601756j,-0.12232531-0.03186272j,-0.12761655+0.10338792j,0.09423824+0.0518973j,0.12750517-0.02130581j,-0.09476786+0.04267315j,0.06665119-0.14795889j,-0.06835935-0.10287437j,-0.10413792+0.08956231j,-0.05873067+0.11992722j,-0.86317563+1.23228226j,-0.00599303-0.15111446j,-0.15767304+0.0869762j,0.15225390+0.02870641j,0.15700377-0.04909256j,0.19282822-0.13294225j,-0.26448426-0.04526069j,0.03933900+0.14146156j,0.15589656+0.01998093j,-0.12850545+0.05866292j,0.15268186+0.05918796j,0.03226310-0.14711876j,0.06187354-0.12063286j,-0.08411862-0.11070104j,-0.11643679+0.13020793j,0.20168803+0.j]
    return(eigs)


def evcal(na, ne, toliter, Vs, V, gains):
	# Load the custom C library routine.
	import ctypes
	_mylib = ctypes.cdll.LoadLibrary("./libevcal.so")
	k = ctypes.c_int()
	return _mylib.evcal_main(
            ctypes.c_int(na),
            ctypes.c_int(ne),
            ctypes.c_double(toliter),
            Vs.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
            V.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
            gains.ctypes.data_as(ctypes.POINTER(ctypes.c_double)),
            ctypes.byref(k) )

def prtime(t):
        """
        convert time of 24hrs integer 
        return (day, hour, min, sec)
        """
        day  = int(t)
        hour = 24 * t
        if hour > 24 and hour < 48:
                hour = hour - 24
        if hour > 48 and hour < 72:
                hour = hour - 48
        if hour > 72:
                print 'CAUTION something strange with the days'
                sys.exit(-1)
        min  = (hour - int(hour))*60
        sec  = (min - int(min))*60
        return (day,int(hour),int(min),sec)


def gettimerange(datab):
    """
    determine the timerange and retunr the time vector
    """
    import copy
    bsl  = datab.keys()   # get the baselines from the database

    mtv = []
    for i in range(len(bsl)):
        bslk = datab[bsl[i]].keys()
        for j in range(len(bslk)):
            tkey = datab[bsl[i]].keys()[j].count('time')
            if tkey > 0:
                if len(datab[bsl[i]][datab[bsl[i]].keys()[j]]) > len(mtv):
                    mtv = []
                    for z in range(len(datab[bsl[i]][datab[bsl[i]].keys()[j]])):
                        mtv.append(datab[bsl[i]][datab[bsl[i]].keys()[j]][z])
    return(mtv)


def getminmaxuvdis(datab):
    """
    determine the maxiumum and minimum UV-distance
    """
    uvdismax = -1E10
    uvdismin = 1E10
    bsl  = datab.keys()   # get the baselines from the database
    for i in range(len(bsl)):
        bslk = datab[bsl[i]].keys()
        for j in range(len(bslk)):
            tkey = datab[bsl[i]].keys()[j].count('uvdis')
            if tkey > 0:
                mav = max(datab[bsl[i]][datab[bsl[i]].keys()[j]])
                miv = min(datab[bsl[i]][datab[bsl[i]].keys()[j]])
                if mav > uvdismax:
                    uvdismax = mav
                if uvdismin > miv:
                    uvdismin = miv
    return(uvdismin,uvdismax)

def vis2matrix(datab,stokes='RR',timestep=0,IF=0):
    """
    place the visibilities into a complex matrix
    """
    import numpy 
    #
    bsl  = datab.keys()   # get the baselines from the database
    # determine number of antennas in the array from the database
    nant = -1               # default for numbers of antennas
    for i in range(len(bsl)):
        if max(eval(bsl[i])) > nant:
                nant = max(eval(bsl[i]))
    # Fill the complex matrix only if the data is valid (weight > 0)
    #
    vism = numpy.zeros([nant,nant],dtype=numpy.complex128)
    for i in range(len(bsl)):
        #
        # print 'PRINT ALL THE BASELINES',datab[bsl[i]].keys()
        # ['imag0RR', 'real0RR', 'time0RR', 'weig0RR', 'uvdis0RR', 'uvw0RR']
        #
        # Careful these arrays could be cubes (time and frequency)
        real = datab[bsl[i]]['real'+str(IF)+stokes]
        imag = datab[bsl[i]]['imag'+str(IF)+stokes]
        weig = datab[bsl[i]]['weig'+str(IF)+stokes]

        if weig[timestep] > 0:
            #
            # indexing according to array size ([0,1] would be baseline [1,2])
            #
            nbsl = eval(bsl[i])
            #
            vism[nbsl[0]-1][nbsl[1]-1] = complex(real[timestep][0],imag[timestep][0])
            # to fill up the entire matix
            vism[nbsl[1]-1][nbsl[0]-1] = complex(real[timestep][0],imag[timestep][0]).conjugate()

    return(vism)


def evdcal(data,sources,ne=1,toliter=1e-7,IF=0):
    """
    here all the calibration starts
    """
    from copy import deepcopy
    import numpy
    import ctypes
    # get stokes parameter
    stokes  = givestokes(data)
    
    # load org database
    datab     = datatobase(data,IF)
    
    # make model visibility
    uvmodel   = mkmodel(data,sources,IF)

    # load mode database
    databmod  = datatobase(uvmodel,IF)

    # get the timerange
    timerange = gettimerange(datab)

    # Not really nice 
    ovismatrix   = vis2matrix(datab,'RR',0,IF)
    gains = numpy.zeros(len(ovismatrix), dtype=numpy.complex128)

    getsoltimestep = 100

    antgains = []
    for t in range(len(timerange)):
            gainst = {}
            for st in range(len(stokes)):
                    ovismatrix   = vis2matrix(datab,stokes[st],t,IF)
                    modvismatrix = vis2matrix(databmod,stokes[st],t,IF)
                    modvismatrix = numpy.ones([len(ovismatrix),len(ovismatrix)], dtype=numpy.complex128)
                    print modvismatrix
                    if getsoltimestep-1 == t:
                        import sys
                        import os
                        print 'orgvis=',ovismatrix
                        print 'modvis=',modvismatrix
                        os.system('rm -fr da00')
                        os.system('rm -fr aipsdisk1')
                        sys.exit(-1)
                    # here checks for errors
                    if evcal(len(ovismatrix), ne, toliter,modvismatrix,ovismatrix,gains) !=0:
                        print 'eval output strange ',evcal(len(ovismatrix), ne, toliter,modvismatrix,ovismatrix,gains)
                        print 'Need more input from Fred (07.04.2011)'

                    eigvalg      = deepcopy(gains)
                    if newisnanv(eigvalg) == True:
                        print 'Eigenvalues are not determine ',eigvalg[-1],
                        print ' at time :',prtime(timerange[t])
    
                    #eigvalg    = eigval(len(ovismatrix),stokes[st]) # old just to test the function
                    for i in range(len(eigvalg)):
                            if gainst.has_key('ANT_'+str(i+1)):
                                    if gainst['ANT_'+str(i+1)].has_key(stokes[st]):
                                            gainst['ANT_'+str(i+1)][stokes[st]].append(eigvalg[i])
                                    else:
                                            gainst['ANT_'+str(i+1)][stokes[st]] = eigvalg[i]
                            else:
                                    gainst['ANT_'+str(i+1)] = {}
                                    if gainst['ANT_'+str(i+1)].has_key(stokes[st]):
                                            gainst['ANT_'+str(i+1)][stokes[st]].append(eigvalg[i])
                                    else:
                                            gainst['ANT_'+str(i+1)][stokes[st]] = eigvalg[i]
            antgains.append([timerange[t],gainst])

    return(antgains)

def checkfi(chfi='None'):
    """
    check if file is present input string 
    will give back 
    value 1 of if present
         -1 not present
    """
    import os
    try:
       # Check path/file is present
       checkfile=1
       os.stat(str(chfi))
    except os.error:
       checkfile=-1
       pass
    return(checkfile)

def filesindir(filext='None',dir='None'):
    """
    returns a vector of the files in the current 
    directory with the given extention
    """
    import os
    from os import stat
    from numpy import sort,take,argsort
    #
    if dir == 'None':
        cwd = os.getcwd()
    files = os.listdir(cwd)
    #
    filesindir = []
    f = ''
    cfilext = []

    for i in range(len(filext)):
        if filext[i] != '*':
          f = f + filext[i]  
        else:
            cfilext.append(f)
            f = ''
    cfilext.append(f)
    cpara = 0

    for i in range(len(files)):
        for j in range(len(cfilext)):
            if files[i].count(cfilext[j]) > 0:
                cpara = cpara + 1
        if cpara == len(cfilext):
            filesindir.append(files[i])
        cpara = 0

    if filext[-1] != '*':
        filesindir3 = filesindir
        filesindir = []
        for i in range(len(filesindir3)):
            if filext[-1] == filesindir3[i][-1]:
                filesindir.append(filesindir3[i])

    # Sort the files by time
    #
    filesindirstat = []
    for i in range(len(filesindir)):
        filesindirstat.append(stat(filesindir[i])[8])

    sortindex = argsort(filesindirstat)
    filesindir2 = take(filesindir,sortindex)
    
    filesindir3 = []
    for i in range(len(filesindir2)):
        filesindir3.append(filesindir2[i])

    return(filesindir3)


def closurepair(nant):
        """
        produce two 1 dimensional arrays for
        phase closure triangels 
        amplitude closure squares
        """
        clp,cla = [],[]
        for i in range(1,nant+1):
                for j in range(i+1,nant+1):
                        for k in range(j+1,nant+1):
                                clp.append([[i,j],[j,k],[i,k]])
                                for l in range(k+1,nant+1):
                                        # for amplitude closure relation see
                                        # see page 192 NRAO  Synthesis 
                                        cla.append([[i,j],[k,l],[i,k],[j,l]]) 
        return(clp,cla)


def loadfile(file,outdata=['EDATA','UVDATA',1,1]):
    """
    """
    import sys
    from AIPSData import AIPSUVData, AIPSImage
    from AIPSTask import AIPSTask
    
    if len(filesindir(file))== 1:
        fitld          = AIPSTask('fitld')
        fitld.outname  = outdata[0]
        fitld.outclass = outdata[1]
        fitld.outseq   = 1
        fitld.outdisk  = int(outdata[3])
        fitld.douvcomp = -1
        fitld.digicor  = -1
        print '\nImport file: ',str(file),' \n\n\n'
        fitld.datain='PWD:'+str(file)
        fitld.go()
        #
        return(outdata)
    else:
        print 'File not present: ',file
        sys.exit(-1)

# ------- some plotting stuff


def pl2ps(data,psname,plver=-1,dpl=-1,domove=1):
    """
    produce ps file from a pl file and delete it afterwards  
    """
    from AIPSTask import AIPSTask
    import os

    if  datatype(data) == 'UV':
        from AIPSData import AIPSUVData as PLData
    else:
        from AIPSData import AIPSImage as PLData

    if plver == -1:
        plver = getntable(data,'PL')

    if plver > 0:
        #
        pldata = PLData(data[0],data[1],data[2],data[3])
        #
        psname = psname+'.ps'
        psoutput = 'GPLOT.ps'
        cwd = os.getcwd()
        #
        lwplah          = AIPSTask('lwpla')
        lwplah.indata   = pldata
        lwplah.plver    = 1
        lwplah.invers   = plver
        lwplah.dparm[5] = 1
        lwplah.lpen     = 1
        lwplah.docolor  = -1
        lwplah.outfile  = 'PWD:'+psoutput
        #lwplah.outfile  = cwd+'/'+psoutput
        lwplah.msgkill  = -2
        lwplah.go()
        #
        if dpl != 0:
            # erase all PL tables
            pldata.zap_table('PL', dpl)

        if domove > 0:
            if len(filesindir(psname)) == 0:
                mvtofilen = 'mv '+cwd+'/'+psoutput+' '+psname
            else:
                mvtofilen = 'mv '+cwd+'/'+psoutput+' '+psname.replace('ps','')+str(len(filesindir(psname)))+'.ps'
                psname = psname.replace('ps','')+str(len(filesindir(psname)))+'.ps'

            os.system(mvtofilen)

        return(psname)
    else:
        return('')


def plotsolution(data,pltname,optype,snver=0):
    """
    plots the SN table (the solution)
    optype should be 'PHAS', 'AMP', 'REAL', 'IMAG'
    """
    from AIPSTask import AIPSTask
    from AIPSData import AIPSUVData

    uvdata  = AIPSUVData(data[0],data[1],data[2],data[3])

    if snver == 0:
        snver = getntable(data,'SN')
    
    snplt         = AIPSTask('SNPLT')
    snplt.indata  = uvdata 
    snplt.inext   = 'SN'
    snplt.inver   =  snver
    snplt.optype  = optype
    snplt.nplots  = 10
    snplt.go()
    #
    return(pl2ps(data,pltname,plver=-1,dpl=-1,domove=1))

def getvisinfo(data,channel=1,headinfo=None):
        #    Calculates the correct value of the pixel
        #    according to email exchange with Eric Geisen 
        #    freq = freq0 + (ch - refch) * increment
        #
        #    channel needs to be 1
        #
        if headinfo == None:
                        headinfo = getheader(data,'FREQ')
        reffreq  = headinfo[0] + (channel - headinfo[1]) * headinfo[2]
        if headinfo[3] >= channel:
                return(reffreq)
        else:
                return(-1)

def comptimev(tiva,tivb):
        """
        """
        indexa,goodata = [],[]
        for ta in range(len(tiva)):
                indexa.append(ta)
                try:
                        tivb.index(tiva[ta])
                except ValueError:
                        goodata.append(-1)
                else:
                        goodata.append(tivb.index(tiva[ta]))
        return indexa,goodata

	
def compdata(dataa,datab,IF=0):

	dbasea,dbaseheadera  = datatobase(dataa,IF,True)
	dbaseb,dbaseheaderb  = datatobase(datab,IF,True)

        obsdate = dbaseheadera['HEADER']['date_obs']

	if dbaseheadera['HEADER']['object'] != dbaseheaderb['HEADER']['object']:
                import sys
		print 'Data object is not the same: ', dbaseheadera['HEADER']['object'], dbaseheaderb['HEADER']['object']
		sys.exit(-1)

	if dbaseheadera['HEADER']['date_obs'] != dbaseheaderb['HEADER']['date_obs']:
                import sys
		print 'Observation data is not the same: ', dbaseheadera['HEADER']['date_obs'], dbaseheaderb['HEADER']['date_obs']
		sys.exit(-1)

	bsla,stokesa,IFa = dbaseinfo(dbasea)
	bslb,stokesb,IFb = dbaseinfo(dbaseb)
	headinfoa        = getheader(dataa,'FREQ')
	headinfob        = getheader(datab,'FREQ')
	#
	nodataonbsl      = []
	nodataonbslwe    = []
	nodataonbslvis   = []
	nodataonbslvisch = []
	#
	for bla in range(len(bsla)):
		if bslb.count(bsla[bla]) > 0:
			bslkeysa = dbasea[bsla[bla]].keys()
			bslkeysb = dbaseb[bsla[bla]].keys()
			for ke in range(len(bslkeysa)):
				if bslkeysa[ke].count('weig') > 0:
					stok      = str(bslkeysb[ke].replace('weig'+str(IF),''))
					if bslkeysb.count(bslkeysa[ke]) > 0:
						tiva      = dbasea[bsla[bla]][str(bslkeysa[ke].replace('weig','time'))]
						tivb      = dbaseb[bsla[bla]][str(bslkeysb[ke].replace('weig','time'))]
						
						index,goodata = comptimev(tiva,tivb)	

						for t in range(len(index)):
							if goodata[t] == -1:
								nodataonbslvis.append([eval(bsla[bla]),stok,tiva[t]])
							else:
								dataaweiga = dbasea[bsla[bla]][str(bslkeysa[ke])][index[t]]
								dataaweigb = dbaseb[bsla[bla]][str(bslkeysa[ke])][goodata[t]]
								#
								chpunitsa = []
								chpunitsb = []
								for ca in range(len(dataaweiga)):
									chpunitsa.append(getvisinfo(dataa,ca+1,headinfoa))

								for cb in range(len(dataaweigb)):
									chpunitsb.append(getvisinfo(dataa,cb+1,headinfob))

								indexca,goodchb = comptimev(chpunitsa,chpunitsb)
								for cia in range(len(indexca)):
									if goodchb[cia] == -1:
										nodataonbslvisch.append([eval(bsla[bla]),stok,tiva[t],indexca[cia]+1])
									else:
										if 0 >= dataaweigb[goodchb[cia]] and dataaweiga[indexca[cia]] > 0:
											nodataonbslvisch.append([eval(bsla[bla]),stok,tiva[t],indexca[cia]+1])
					else:
						nodataonbslwe.append([eval(bsla[bla]),stok])
		else:
			nodataonbsl.append(eval(bsla[bla]))
			
	return(obsdate,nodataonbsl,nodataonbslwe, nodataonbslvis, nodataonbslvisch)

def uvflg_textinput(fgant,fgbas,fgtim,fgsto,fgbchan,fgechan,fbif,feif,fgreas,filename):

        vlen = [len(fgant),len(fgbas),len(fgtim),len(fgsto),len(fgbchan),len(fgechan),len(fbif),len(feif)]
        if min(vlen) != max(vlen):
                import sys
                print 'Error some of the vectors have different lengh'
                sys.exit(-1)

                
        if len(fgant)> 0:
                pextf = '.FG.UVFLG'
                # Produce an Text file that can be loaded via UVFLG
                fgfile  = open(filename+pextf,'w')
                print >> fgfile, '! HRK product'
                print >> fgfile, '! '
                print >> fgfile, '! This may be edited if you understand what you are doing'
                print >> fgfile, '! '
                print >> fgfile, '! '
                for i in range(len(fgant)):
                        fgt1 = prtimefg(fgtim[i][0],'low')
                        fgt2 = prtimefg(fgtim[i][1],'up')
                        fgtr = str(fgt1[0])+','+str(fgt1[1])+','+str(fgt1[2])+','+str(fgt1[3])+\
                                ','+str(fgt2[0])+','+str(fgt2[1])+','+str(fgt2[2])+','+str(fgt2[3])
                        print >> fgfile,'TIMERANG='+fgtr,
                        print >> fgfile,' ANTENNAS='+str(fgant[i]),
                        print >> fgfile,' BASELINE='+str(fgbas[i]),
                        print >> fgfile,' BCHAN='+str(fgbchan[i]),
                        print >> fgfile,' ECHAN='+str(fgechan[i]),
                        print >> fgfile,' BIF='+str(fbif[i]),
                        print >> fgfile,' EIF='+str(feif[i]),
                        print >> fgfile,' STOKES='+"'"+str(fgsto[i])+"'",
                        if len(fgreas) > 2:
                                print >> fgfile,' REASON='+"'"+str(fgreas[i])+"'",
                        else:
                                print >> fgfile,' REASON='+"'"+str(fgreas[0])+"'",

                        print >> fgfile,'/'

                return(filename+pextf)
        return('')

def fgcmd_textinput(obsd,fgant,fgbas,fgtim,fgsto,fgbchan,fgechan,fbif,feif,fgreas,filename):

        vlen = [len(fgant),len(fgbas),len(fgtim),len(fgsto),len(fgbchan),len(fgechan),len(fbif),len(feif)]
        if min(vlen) != max(vlen):
                import sys
                print 'Error some of the vectors have different lengh'
                sys.exit(-1)

                
        if len(fgant)> 0:
                pextf = '.FG.FGCMD'
                # Produce an Text file that can be loaded via UVFLG
                fgfile  = open(filename+pextf,'w')
                #print >> fgfile, '! HRK product'
                #print >> fgfile, '! '
                #print >> fgfile, '! This may be edited if you understand what you are doing'
                #print >> fgfile, '! '
                #print >> fgfile, '! '
                for i in range(len(fgant)):
                        fgt1 = prtimefg(fgtim[i][0],'low')
                        fgt2 = prtimefg(fgtim[i][1],'up')

                        # need to be checked because of day +1
                        # 
                        #fgtr = "'"+str(obsd[0])+'/'+str(obsd[1])+'/'+str(obsd[2])+\
                        #    '/'+str(fgt1[1])+':'+str(fgt1[2])+':'+str(fgt1[3])+'~'+\
                        #    str(obsd[0])+'/'+str(obsd[1])+'/'+str(obsd[2])+\
                        #    '/'+str(fgt2[1])+':'+str(fgt2[2])+':'+str(fgt2[3])+"'"


                        fgtr = "'"+str(obsd[0])+'/'+str(obsd[1])+'/'+str(obsd[2]+fgt1[0])+\
                            '/'+str(fgt1[1])+':'+str(fgt1[2])+':'+str(fgt1[3])+'~'+\
                            str(obsd[0])+'/'+str(obsd[1])+'/'+str(obsd[2]+fgt2[0])+\
                            '/'+str(fgt2[1])+':'+str(fgt2[2])+':'+str(fgt2[3])+"'"
                        print >> fgfile,'timerange='+fgtr,
                        print >> fgfile,' antenna='+"'"+str(fgant[i])+'&'+str(fgbas[i])+"'",
                        print >> fgfile,' spw='+"'"+'0:'+str(fgbchan[i])+'~'+str(fgechan[i])+"'",
                        print >> fgfile,' correlation='+"'"+str(fgsto[i])+"'"
                        #print >> fgfile,'/'

                return(filename+pextf)
        return('')

def dbasetimer(database,stokes='RR'):
        """
	determine the maximum timerange in a database
        """
	from numpy import sort
        #
	bsl,dbstokes,IF = dbaseinfo(database)
     	#
	if dbstokes.count(stokes) > 0:
		st = dbstokes.index(stokes)
	#
	ntime = []
        #
        for bl in range(len(bsl)):
                if database[bsl[bl]].has_key('real'+str(IF)+dbstokes[st]):
                        time = database[bsl[bl]]['time'+str(IF)+dbstokes[st]]
                        for j in range(len(time)):
                                if ntime.count(time[j]) == 0:
                                        ntime.append(time[j])
        time  = []
        stime = sort(ntime)
	for i in range(len(stime)):
            time.append(stime[i])

        return(time)

def prtimefg(t,blowup=None):
        """
        convert time of 24hrs integer 
        return (day, hour, min, sec)
        """
        from math import floor,ceil
        
        day  = floor(t)

        hour = 24 * (t - floor(t))
        min  = (hour - floor(hour))*60
        sec  = (min - floor(min))*60
        if blowup == 'low':
            sec  = floor((min - floor(min))*60)
        if blowup == 'up':
            sec  = ceil((min - floor(min))*60)
        return (int(day),int(hour),int(min),sec)

def detinttime(UVDAT,intfilename=None,IF=1,**kwargs):
	"""
	determine the integration time of the UVDATA set
	[s]
	"""
	import os
	from numpy import array

	if intfilename != None:
		if checkfi(intfilename) > 0:
			print '- Information about integration time exists in file',intfilename,'\n'
			os.system('/bin/cp '+str(intfilename)+' INTTIME.py')
			import INTTIME
			if checkfi('INTTIME.py') > 0:
				reload(INTTIME)
			else:
				os.system('/bin/cp '+str(intfilename)+' INTTIME.py')
				reload(INTTIME)
			os.system('/bin/rm -fr  INTTIME.py*')	
			return(INTTIME.integrationtime)
		
	integrationtime = -1
	# check if data is present
	aipsdata(UVDAT)
	# Determine the integrationtime
	nbsl     = baselines(getnantennas(UVDAT))
	stokes   = givestokes(UVDAT)

        if kwargs.has_key('dbase') == True:
            database = kwargs['dbase']
        else:
            database = datatobase(UVDAT)

	from copy import copy

	mindt  = []
	for st in range(len(stokes)):
		tkey = 'time'+str(IF-1)+stokes[st]
		for b in range(len(nbsl)):
			if database.has_key(str(nbsl[b])) == True:
				#
				if database[str(nbsl[b])].has_key(tkey) == True:
					time  = array(copy(database[str(nbsl[b])][tkey]))
					dtime = []
					for i in range(len(time)-1):
						dtime.append(time[i+1]-time[i])
					if len(dtime) > 0:
						mindt.append(min(dtime)*24.0*60.0*60.0)
	
	mindt = delzero(mindt)
	mindt = delzero(mindt,0.0)
	#
	integrationtime = min(mindt)

	if intfilename == None:
		return(integrationtime)		
	else:
		intfile = open(intfilename,'w')
		print >> intfile, 'integrationtime = ',integrationtime
		intfile.close()
		return(integrationtime)

def datatype(data):
    """
    return the datatype of the file
    """
    from AIPSData import AIPSCat
    from string import split,find
    #
    datatype = 'None'
    
    catalog_in = AIPSCat(int(data[3]))
    catalog    = str(catalog_in).split()
    #
    matchfname = []
    for i in range(len(catalog)):
        if catalog[i].find(data[0]) == 0:
                matchfname.append(i)
    matchfname.append(len(catalog))

    classindex = -1
    isname  = -1
    isclass = -1
    isdisk  = -1
    isseq   = -1
    i = 0
    while (isname != 1 or isclass != 1 or isdisk !=1 or isseq !=1 or classindex != -1) and i < len(matchfname)-1:
        isname  = -1
        isclass = -1
        isdisk  = -1
        isseq   = -1
        #print matchfname[i],matchfname[i+1]
        #print catalog[matchfname[i]:matchfname[i+1]]
        for j in range(matchfname[i],matchfname[i+1]):

            if catalog[j].replace('.','') == data[0].replace('.',''):
                isname = 1
            if catalog[j].replace('.','') == data[1]:
                isclass = 1
            if catalog[j].replace('.','') == str(int(data[2])):
                isdisk = 1
            if catalog[j].replace('.','') == str(int(data[3])):
                isseq = 1
                        
            if catalog[j] == 'UV' and isname == 1 and isclass == 1 and isdisk == 1 and isseq ==1 and classindex == -1:
                classindex = j
            if catalog[j] == 'MA' and isname == 1 and isclass == 1 and isdisk == 1 and isseq ==1 and classindex == -1:
                classindex = j
        i += 1

    if classindex != -1:
        datatype = catalog[classindex]

    return(datatype)


def aipsdata(data,exit=False):
    """
    aborts the program if the file is not present in the AIPS disk
    data vector in PT convention to access the file 
    [NAME, CLASS, DISK, SEQ]
    """
    if  datatype(data) == 'UV':
        from AIPSData import AIPSUVData as CHECKData
    else:
       from AIPSData import AIPSImage as CHECKData
    #
    chdata = CHECKData(data[0],data[1],int(data[2]),int(data[3]))
    if chdata.exists() == False:
        print 'Caution ',data,' does not exists !' 
        if exit == True:
            sys.exit(-1)
        else:
            return(-1)
    else:
        return(1)

def getnantennas(data):
	"""
	returns the number of speficied antennas 
	"""
	from Wizardry.AIPSData import AIPSUVData as AIPSDataTAB

	tabdata   = AIPSDataTAB(data[0],data[1],data[2],data[3])
	
	tabn = getntable(data,'AN')

	if tabn > 0:
		nant = len(tabdata.table('AN',tabn))
	else:
		nant = -1
	return(nant)

def delzero(data,value=0):
	"""
	will retrun vector with values != value
	"""
	newvec = []
	for a in range(len(data)):
		if data[a] != value:
			newvec.append(data[a])
	return(newvec)

def baselines(antennas):
    """
    returns the baseline pairs
    """
    ant = range(1,antennas+1)
    baselpair = [ [i,j] for i in ant for j in ant if i<j ];
    return(baselpair)
