# HRK 2012/175
#
#
# The purpose of this program is to compare two UV datasets
#   - one is the original e.g. multi source data
#   - one is the final flagged and calibrated data
#   and determine all the data that has been flagged with respect to the original data
#
# the output of this program can either be a UVFLG file readable via the AIPS UBFLAG
# or the outout is a flagcmd ascii file that can be loaded into the CASA task flagcmd
#
# usage:
#       python UVFITSFG_2_FLAGCMD_TXT.py 3C147.UV.FITS 3C147.FG.UV.FITS
#
#
#  in CASA:
#          load data into CASA with:
#                                  task importuvfits
#                                  fitsfile='3C147.UV.FITS'
#                                  vis='test.ms'
#                                  go
#
#          apply the flagging with:
#                                  task flagcmd
#                                  vis='test.ms'
#                                  flagmode='file'
#                                  flagfile='3C147_FLAGCMD.FG.FGCMD'
#                                  go
# ##


# Author: 

# Hans - Rainer Kloeckner
# MPIfR / Oxford
# hrk@mpifr-bonn.mpg.de
# hrk@astro.ox.ac.uk

# ---------------------------------------------------
#
# requirements:
#             AIPS, PARSELTongue, numpy
#
# files required :
#                 AIPSLite.py, AIPSLiteTask.py, HRKinitAIPS.py, COMP_LIB.py            
#
# ##############################################################################
# ##############################################################################


# Copyright (c) 2012
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>,
# or write to the Free Software Foundation, Inc., 
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# import some general libs
import sys
import os


# ----SETTINGS------------------------------------------------------------------

#
# AIPS user number 
aipsid       = 815
#
# AIPSDISK directory
aipsdiskname = 'aipsdisk1'

#
# Do some logistic settings
sselcalog    = 'SCPRT'
#
# Define the input/output disk
indata       = 1
outdata      = indata
#

# if True writes out FLAGCMD ascii input text file (CASA)
doflagcmdout = True

# if True writes out UVFLAG ascii input text file  (AIPS)
douvflagout  = False

# deletes the AIPS directories
docleanup    = True   
#
# ##############################################################################
# ##############################################################################
# ##############################################################################
#
# Initialise AIPS disk 
#
# make the AIPS directories 
#
from HRKinitAIPS import *
initAIPSLite(aipsdiskname)

# get the COMP lib
from COMP_LIB import *

# Get the PT environment
#
from AIPS import AIPS
from AIPSTask import AIPSTask, AIPSList
from AIPSData import AIPSUVData, AIPSImage,AIPSCat
from Wizardry.AIPSData import AIPSUVData as WizAIPSUVData
#
# ##############################################################################

#  Some additional AIPS internal settings
#
AIPS.userno  =  aipsid
AIPSCat(1)   # This is needed to produce some AIPS files in the directories
#
# INITIALISE LOG
selcalog      = sselcalog+'.log'
AIPS.log      = open('./'+selcalog,'a')


# Load the data file
#
data  = loadfile(sys.argv[1],['EDATA','UVDAT',1,1])
data2 = loadfile(sys.argv[2],['EDATA2','UVDAT2',1,1])


# needs a further copy because of some GMRT related stuff 
#
uvdata           = AIPSUVData(data[0],data[1],data[2],data[3])
#
uvcop             = AIPSTask('uvcop')
uvcop.indata      = uvdata
uvcop.outname     = 'tmp1'
uvcop.outclass    = 'BSLDAT'
uvcop.outdisk     = int(outdata)
uvcop.go()
#
data              = ['tmp1','BSLDAT',1,1]
#
# deletes the first input file
uvdata.zap()
# ##

# Check if 1st input data is a multi source file 
# via the header information
#
uvdata            = AIPSUVData('tmp1','BSLDAT',1,1)
orgsource         = uvdata.header['object']

uvdata2           = AIPSUVData('EDATA2','UVDAT2',1,1)
tsource           = uvdata2.header['object']
# ##

if orgsource == 'MULTI':
	print '\nFound multi source file will extract: ',tsource
	splat             = AIPSTask('splat')
	splat.indata      = uvdata
	splat.source[1]   = tsource
	splat.outname     = 'tmp1s'
	splat.outclass    = 'BSLDAT'
	splat.outdisk     = int(outdata)
	splat.go()
	#
	data = ['tmp1s','BSLDAT',1,1]
	#
	# deletes the multi source file
	uvdata.zap()
	
# number of IF (hard coded assume only one IF, otherwise need to edit)
IF = 1

# get stokes parameter
stokes  = givestokes(data)


## DELETE THE BLOCK IN THE FUTURE TO COMPARE ALL THE DATA
##
##
#uvdata           = AIPSUVData(data[0],data[1],data[2],data[3])
##
#uvcop             = AIPSTask('uvcop')
#uvcop.indata      = uvdata
#uvcop.outname     = 'tmp12'
#uvcop.outclass    = 'BSLDAT'
#uvcop.antenna[1]  = 1
#uvcop.baseline[1] = 2
#uvcop.outdisk     = int(outdata)
#uvcop.go()
##
#uvdata2           = AIPSUVData(data2[0],data2[1],data2[2],data2[3])
#uvcop             = AIPSTask('uvcop')
#uvcop.indata      = uvdata2
#uvcop.outname     = 'tmp2'
#uvcop.outclass    = 'BSLDAT'
#uvcop.antenna[1]  = 1
#uvcop.baseline[1] = 2
#uvcop.outdisk     = int(outdata)
#uvcop.go()
##
##
#bsld1 = ['tmp12','BSLDAT',1,1]
#bsld2 = ['tmp2','BSLDAT',1,1]
#
# ##
bsld1  = data 
bsld2  = data2
#

#  convert data into baseline based python dic
#
dbase1 = datatobase(bsld1)

# Determine the maximal timerange
#
mmtimer = []
for st in range(len(stokes)):
	mmtimer += dbasetimer(dbase1,stokes[st])

mintime = min(mmtimer)
maxtime = max(mmtimer)

# Determine maximal channel range
#
minchan = 0    # for AIPS 0 indicates the entire range
maxchan = 0
if doflagcmdout:
	maxchan  = getheader(bsld1,'FREQ')[-1]
	minchan  = 1

# Determine the integration time per visibility needed to be taken
#  into account to get centre of the time stamp for correct flagging
ndt     = detinttime(bsld1,'integrationtime.py',dbase=dbase1)/(2.*24.*60.*60.)

# Here the actual comparsm takes place 
#
obsdate,nodataonbsl,nodataonbslwe, nodataonbslvis,nodataonbslvisch = compdata(bsld1,bsld2)


# Now get the information into the right structure
#
nant,nbas,nfgt,nfgst,bchfg,echfg = [],[],[],[],[],[]
bchfg,echfg = [],[]
bifg,eifg   = [],[]
reason      = []
#
for st in range(len(stokes)):
	for i in range(len(nodataonbsl)):
		nant.append(nodataonbsl[i][0])
		nbas.append(nodataonbsl[i][1])
		nfgt.append([mintime-ndt,maxtime+ndt])
		nfgst.append(stokes[st])
		bchfg.append(minchan)
		echfg.append(maxchan)
		bifg.append(IF)
		eifg.append(IF)
		reason.append('nodataonbsl')
		
for i in range(len(nodataonbslwe)):
		nant.append(nodataonbslwe[i][0][0])
		nbas.append(nodataonbslwe[i][0][1])
		nfgt.append([mintime-ndt,maxtime+ndt])
		nfgst.append(nodataonbslwe[i][1])
		bchfg.append(minchan)
		echfg.append(maxchan)
		bifg.append(IF)
		eifg.append(IF)
		reason.append('nodataonbslwe')

for i in range(len(nodataonbslvis)):
		nant.append(nodataonbslvis[i][0][0])
		nbas.append(nodataonbslvis[i][0][1])
		nfgt.append([nodataonbslvis[i][2]-ndt,nodataonbslvis[i][2]+ndt])
		nfgst.append(nodataonbslvis[i][1])
		bchfg.append(minchan)
		echfg.append(maxchan)
		bifg.append(IF)
		eifg.append(IF)
		reason.append('nodataonbslvis')

for i in range(len(nodataonbslvisch)):
		nant.append(nodataonbslvisch[i][0][0])
		nbas.append(nodataonbslvisch[i][0][1])
		nfgt.append([nodataonbslvisch[i][2]-ndt,nodataonbslvisch[i][2]+ndt])
		nfgst.append(nodataonbslvisch[i][1])
		bchfg.append(nodataonbslvisch[i][3])
		echfg.append(nodataonbslvisch[i][3])
		bifg.append(IF)
		eifg.append(IF)
		reason.append('nodataonbslvisch')
# 
##

# write the flad information into ASCII output
if douvflagout:
	fgf = uvflg_textinput(nant,nbas,nfgt,nfgst,bchfg,echfg,bifg,eifg,reason,filename=tsource+'_UVFLAG')

if doflagcmdout:
	spobs = obsdate.replace('-',' ').split()
	nobs = []
	for t in range(len(spobs)):
		nobs.append(eval(spobs[t]))

	fgf = fgcmd_textinput(nobs,nant,nbas,nfgt,nfgst,bchfg,echfg,bifg,eifg,reason,filename=tsource+'_FLAGCMD')
	
print 'The produced FG file ',fgf,'\nmay be applied to the input file ',sys.argv[1] 

# Clean up the directory
#
if docleanup:
	os.system('rm -fr da00')
	os.system('rm -fr '+aipsdiskname)
